// $Id: Save.java 15994 2012-07-04 16:39:40Z mike $

package org.faceless.pdf2.viewer2.feature;

import org.faceless.pdf2.viewer2.*;
import org.faceless.pdf2.viewer2.util.DialogPanel;
import org.faceless.pdf2.*;
import javax.swing.*;
import javax.swing.plaf.basic.BasicFileChooserUI;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.event.*;
import java.awt.*;
import java.beans.*;
import java.security.*;
import java.awt.event.*;
import java.util.Timer;
import java.util.*;
import javax.swing.filechooser.FileFilter;
import java.io.*;

/**
 * Create a button that will open a dialog allowing the PDF to be saved to disk.
 * Any {@link Exporter} formats included as features in the Viewer will be presented
 * as options.
 * <div class="initparams">
 * The following <a href="../doc-files/initparams.html">initialization parameters</a> can be specified to configure this feature.
 * <table>
 * <tr><th>promptOnOverwrite</th><td>true to prompt before overwriting files, false otherwise (the default)</td></tr>
 * </table>
 *
 * <span class="featurename">The name of this feature is <span class="featureactualname">Save</span></span>
 * <p><i>This code is copyright the Big Faceless Organization. You're welcome to use, modify and distribute it in any form in your own projects, provided those projects continue to make use of the Big Faceless PDF library.</i></p>
 * @since 2.8
 */
public class Save extends ViewerWidget implements DocumentPanelListener {

    private boolean overwriteprompt = false;

    public Save() {
        super("Save");
        setButton("Document", "resources/icons/disk.png", "PDFViewer.tt.Save");
        setMenu("File\tSave", 's');
    }

    public boolean isEnabledByDefault() {
        return Util.hasFilePermission();
    }

    public void initialize(PDFViewer viewer) {
        super.initialize(viewer);
                 
        String val = getFeatureProperty(viewer, "promptOnOverwrite");
        if (val != null) {
            setPromptOnOverwrite("true".equals(val));
        }

        viewer.addDocumentPanelListener(this);
    }

    public void documentUpdated(DocumentPanelEvent event) {
        if (event.getType() == "activated" && event.getDocumentPanel().getPDF() != null) {
            EncryptionHandler handler = event.getDocumentPanel().getPDF().getEncryptionHandler();
            boolean enabled = handler==null || handler.hasRight("Save");
            event.getDocumentPanel().getViewer().getNamedComponent("ButtonSave").setEnabled(enabled);
            event.getDocumentPanel().getViewer().getNamedComponent("MenuSave").setEnabled(enabled);
        }
    }

    /**
     * Set whether this feature should prompt before overwriting a file
     * @param prompt whether to prompt before overwriting a file (detault is false)
     * @since 2.11.25
     */
    public void setPromptOnOverwrite(boolean prompt) {
        this.overwriteprompt = prompt;
    } 

    public void action(final ViewerEvent event) {
        save(event, null, null, true, overwriteprompt);
    }

    /**
     * Save the Document
     * @param event the ViewerEvent that launched this action
     * @param initialexporter the {@link Exporter} to choose by default, or null to default
     * to a {@link PDFExporter}
     * @param initialpath the Path to display by default, or <code>null</code> to use the same
     * path as the source file
     * @param displayprompt whether to prompt the user for a filename. Currently ignored.
     * @param overwriteprompt whether to prompt the user if we are about to overwrite a filename.
     * @since 2.11.10
     */
    public static void save(final ViewerEvent event, final Exporter initialexporter, final String initialpath, final boolean displayprompt, final boolean overwriteprompt) {
        AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                final DocumentPanel docpanel = event.getDocumentPanel();
                final PDFViewer viewer = event.getViewer();
                final Exporter[] chosenexporter = new Exporter[] { initialexporter };
                final Map filters = new LinkedHashMap();

                File origfile = null;
                if (initialpath!=null) {
                    origfile = new File(initialpath);
                } else {
                    origfile = (File)docpanel.getClientProperty("file");
                }
                if (origfile==null) {                   // Slightly over paranoid, there appear to
                    try {                               // be Windows bugs in this area.
                        origfile = new File(System.getProperty("user.dir"));
                    } catch (SecurityException e) {
                        origfile = File.listRoots()[0];
                    }
                }
                final JFileChooser chooser = new JFileChooser(origfile);

                FileFilter firstfilter = null;
                final PDF pdf = event.getPDF();
                ViewerFeature[] features = event.getViewer().getFeatures();
                for (int i=0;i<features.length;i++) {
                    if (features[i] instanceof Exporter) {
                        Exporter exporter = (Exporter)features[i];
                        if (exporter.isEnabled(pdf)) {
                            FileFilter filter = exporter.getFileFilter();
                            if (firstfilter==null || exporter==initialexporter || (initialexporter==null && exporter instanceof PDFExporter)) {
                                firstfilter = filter;
                            }
                            chooser.addChoosableFileFilter(filter);
                            filters.put(filter, exporter);
                        }
                    }
                }
                if (filters.size()==0) {                // No filters - always add a PDFExporter
                    Exporter ex = new PDFExporter();
                    firstfilter = ex.getFileFilter();
                    chooser.addChoosableFileFilter(firstfilter);
                    filters.put(ex, firstfilter);
                }
                chooser.addPropertyChangeListener(new PropertyChangeListener() {
                    public void propertyChange(PropertyChangeEvent event) {
                        if (event.getPropertyName().equals(chooser.FILE_FILTER_CHANGED_PROPERTY)) {
                            Exporter ex = (Exporter)filters.get(chooser.getFileFilter());
                            if (ex!=null) {
                                chosenexporter[0] = ex;
                                if (chooser.getUI() instanceof BasicFileChooserUI) {
                                    BasicFileChooserUI ui = (BasicFileChooserUI)chooser.getUI();
                                    String name = ui.getFileName();
                                    int ix = name==null ? -1 : name.lastIndexOf(".");
                                    if (ix>0 && name.length()-ix<5) {
                                        name = name.substring(0, ix+1)+ex.getFileSuffix();
                                        ui.setFileName(name);
                                        if (chooser.getSelectedFile()!=null) {
                                            chooser.setSelectedFile(new File(chooser.getSelectedFile(), name));
                                        }
                                    }
                                } else {
                                    File file = chooser.getSelectedFile();
                                    if (file!=null) {
                                        String name = file.getName();
                                        int ix = name==null ? -1 : name.lastIndexOf(".");
                                        if (ix>0 && name.length()-ix<5) {
                                            name = name.substring(0, ix+1)+ex.getFileSuffix();
                                            chooser.setSelectedFile(new File(file, name));
                                        }
                                    }
                                }
                            }
                        }
                    }
                });
                chooser.setFileFilter(firstfilter);
                chooser.setSelectedFile(origfile);

                if (chooser.showSaveDialog(viewer)==chooser.APPROVE_OPTION) {
                    final File targetfile = chooser.getSelectedFile();
                    if (targetfile.exists() && !targetfile.canWrite()) {
                        String msg = Util.getUIString("PDFViewer.ReadOnly", targetfile.getName());
                        JOptionPane.showMessageDialog(viewer, msg, UIManager.getString("PDFViewer.Error"), JOptionPane.ERROR_MESSAGE);
                    } else {
                        if (!targetfile.exists() || !overwriteprompt || JOptionPane.showConfirmDialog(viewer, UIManager.getString("PDFViewer.OverwriteFile"), UIManager.getString("PDFViewer.OverwriteFile"), JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION) {
                            final Exporter exporter = chosenexporter[0];
                            final JComponent secondarycomp = exporter.getComponent();

                            boolean ok;
                            if (secondarycomp != null) {
                                DialogPanel dialog = new DialogPanel() {
                                    public String validateDialog() {
                                        return exporter.validateComponent(secondarycomp);
                                    }
                                };
                                secondarycomp.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
                                dialog.addComponent(secondarycomp);
                                dialog.setButtonText("ok", UIManager.getString("PDFViewer.Save"));
                                ok = dialog.showDialog(viewer, Util.getUIString("PDFViewer.SavingFile", "PDF"));
                            } else {
                                ok = true;
                            }
                            if (ok) {
                                try {
                                    // If the file we're saving to exists, render to a temporary file in the
                                    // same directory and rename. If it's the same file we've read from,
                                    // we need to ensure file backing store remains uncorrupted. We do this
                                    // by setting the "MultipleRevisions" feature to required. This will
                                    // preserve the file structure for File based PDF (and do nothing otherwise)
                                    //
                                    pdf.getBasicOutputProfile().clearRequired(OutputProfile.Feature.MultipleRevisions);

                                    final File tempfile;
                                    if (targetfile.exists()) {
                                        String tempname = targetfile.getName();
                                        tempname = tempname.substring(0, tempname.lastIndexOf(".") + 1);
                                        tempfile = File.createTempFile(tempname, null, targetfile.getParentFile());
                                        if (targetfile.getCanonicalFile().equals(origfile.getCanonicalFile())) {
                                            pdf.getBasicOutputProfile().setRequired(OutputProfile.Feature.MultipleRevisions);
                                        }
                                    } else {
                                        tempfile = targetfile;
                                    }

                                    final OutputStream out = new FileOutputStream(tempfile);
                                    Exporter.ExporterTask task = chosenexporter[0].getExporter(docpanel, docpanel.getPDF(), secondarycomp, out);
                                    task.addPropertyChangeListener(new PropertyChangeListener() {
                                        public void propertyChange(PropertyChangeEvent event) {
                                            if (event.getPropertyName().equals("state") && "running".equals(event.getOldValue())) {
                                                if (tempfile!=targetfile) {         // If we're writing to a temp file
                                                    if ("cancelled".equals(event.getNewValue())) {
                                                        try {
                                                            out.close();
                                                        } catch (IOException e) {
                                                            Util.displayThrowable(e, viewer);
                                                        }
                                                        if (!tempfile.delete()) {
                                                            tempfile.deleteOnExit();
                                                        }
                                                    } else if ("completed".equals(event.getNewValue())) {
                                                        docpanel.getPDF().close();
                                                        boolean windows = System.getProperty("os.name").startsWith("Windows");
                                                        if ((windows && !targetfile.delete()) || !tempfile.renameTo(targetfile)) {
                                                            Exception e = new IOException("Couldn't rename \""+tempfile+"\" to \""+targetfile+"\"");
                                                            Util.displayThrowable(e, viewer);
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    });
                                    task.start(viewer, Util.getUIString("PDFViewer.SavingFile", targetfile.toString()));
                                } catch (IOException e) {
                                    Util.displayThrowable(e, viewer);
                                }
                            }
                        }
                    }
                }
                return null;
            }
        });
    }
}
